from five import grok
from euphorie.client.country import View as BaseView
from euphorie.client.country import IClientCountry
from tno.euphorie.interfaces import ITnoClientSkinLayer

grok.templatedir("templates")

class View(BaseView):
    grok.context(IClientCountry)
    grok.require("euphorie.client.ViewSurvey")
    grok.layer(ITnoClientSkinLayer)
    grok.template("sessions")




from collective.testcaselayer import ptc
from euphorie.deployment.tests.functional import EuphorieLayer
from euphorie.deployment.tests.functional import EuphorieFunctionalTestCase
from Products.PloneTestCase import PloneTestCase


class TnoEuphorieTestLayer(ptc.BasePTCLayer):
    def afterSetUp(self):
        from Testing.ZopeTestCase import installPackage
        import tno.euphorie

        self.loadZCML('configure.zcml', package=tno.euphorie)
        installPackage('tno.euphorie')
        self.addProduct('tno.euphorie')
        # Reinstalling tno.euphorie zaps the membrane_tool contens, so manually
        # reindex the client.
        self.portal.membrane_tool.indexObject(self.portal.client)

        import tno.euphorie.model
        assert tno.euphorie.model._instrumented
        from euphorie.client import model
        from z3c.saconfig import Session
        model.metadata.create_all(Session.bind, checkfirst=True)

    def beforeTearDown(self):
        pass


TnoEuphorieLayer = TnoEuphorieTestLayer([EuphorieLayer, ptc.ptc_layer])


class TnoEuphorieTestCase(PloneTestCase.PloneTestCase):
    layer = TnoEuphorieLayer


class TnoEuphorieFunctionalTestCase(EuphorieFunctionalTestCase):
    layer = TnoEuphorieLayer


def registerUserInClient(browser):
    """Register a new user in the client. This is pretty much a direct copy
    of :py:func:`euphorie.client.tests.utils.registerUserInClient`, but updated
    to use Dutch language for button labels.
    """
    browser.getLink('Registreer').click()
    browser.getControl(name='email').value = 'guest@example.com'
    browser.getControl(name='password1:utf8:ustring').value = 'guest'
    browser.getControl(name='password2:utf8:ustring').value = 'guest'
    browser.getControl(name='next', index=1).click()



from zope.component import adapts
from zope.interface import directlyProvidedBy
from zope.interface import directlyProvides
from zope.publisher.interfaces.browser import IBrowserSkinType
from ZPublisher.BaseRequest import DefaultPublishTraverse
from tno.euphorie.interfaces import IProductLayer
from tno.euphorie.interfaces import ITnoClientSkinLayer
from tno.euphorie.interfaces import ITnoClientAPISkinLayer
from euphorie.client.client import IClient
from euphorie.client.api.entry import access_api


class ClientPublishTraverser(DefaultPublishTraverse):
    """Publish traverser to setup the skin layer.

    This traverser marks the request with ITnoClientSkinLayer when the
    client is traversed and the tno.euphorie product is installed.
    """
    adapts(IClient, IProductLayer)

    def publishTraverse(self, request, name):
        from euphorie.client.utils import setRequest
        setRequest(request)
        request.client = self.context

        if name == 'api':
           api = access_api(request).__of__(self.context)
           directlyProvides(request, ITnoClientAPISkinLayer, [])
           return api

        ifaces = [iface for iface in directlyProvidedBy(request)
                  if not IBrowserSkinType.providedBy(iface)]
        directlyProvides(request, ITnoClientSkinLayer, ifaces)
        return super(ClientPublishTraverser, self)\
                .publishTraverse(request, name)



import datetime
import decimal
from five import grok
from zope.interface import directlyProvides
from zope import schema
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
from plone.directives import form
from plonetheme.nuplone.z3cform.form import FieldWidgetFactory
from euphorie.client.company import Company as GenericCompany
from euphorie.client.session import SessionManager
from euphorie.client.survey import PathGhost
from tno.euphorie.interfaces import ITnoReportPhaseSkinLayer
from tno.euphorie.model import DutchCompany


grok.templatedir("templates")


TextSpan1 = FieldWidgetFactory("z3c.form.browser.text.TextFieldWidget", klass="span-1")
TextSpan6 = FieldWidgetFactory("z3c.form.browser.text.TextFieldWidget", klass="span-6")


class DutchCompanySchema(form.Schema):
    title = schema.TextLine(
            title=u"Bedrijfsnaam",
            max_length=128,
            required=False)

    address_visit_address = schema.TextLine(
            title=u"Adres",
            required=False)
    address_visit_postal = schema.TextLine(
            title=u"Postcode",
            max_length=16,
            required=False)
    address_visit_city = schema.TextLine(
            title=u"Plaats",
            max_length=64,
            required=False)

    address_postal_address = schema.TextLine(
            title=u"Adres",
            required=False)
    address_postal_postal = schema.TextLine(
            title=u"Postcode",
            max_length=16,
            required=False)
    address_postal_city = schema.TextLine(
            title=u"Plaats",
            max_length=64,
            required=False)
    email = schema.ASCIILine(
            title=u"E-mailadres",
            max_length=128,
            required=False)
    form.widget(email="tno.euphorie.company.TextSpan6")
    phone = schema.ASCIILine(
            title=u"Telefoonnummer",
            max_length=32,
            required=False)
    activity = schema.TextLine(
            title=u"Bedrijfsactiviteit",
            max_length=64,
            required=False)
    submitter_name = schema.TextLine(
            title=u"Naam invuller",
            max_length=64,
            required=False)
    submitter_function = schema.TextLine(
            title=u"Functie invuller",
            max_length=64,
            required=False)
    department = schema.TextLine(
            title=u"Afdeling",
            max_length=64,
            required=False)
    location = schema.TextLine(
            title=u"Lokatie",
            max_length=64,
            required=False)
    submit_date = schema.Date(
            title=u"Datum",
            description=u"Datum waarop de gegevens verzameld zijn",
            min=datetime.date(2000,1,1),
            required=False)
    employees = schema.Choice(
            title=u"Aantal werknemers",
            vocabulary=SimpleVocabulary([
                SimpleTerm(u"40h", title=u"Maximaal 40 uur betaalde arbeid per week"),
                SimpleTerm(u"max25", title=u"Maximaal 25 werknemers"),
                SimpleTerm(u"over25", title=u"Meer dan 25 werknemers"),
                ]),
            required=False)
    absentee_percentage = schema.Decimal(
            title=u"Verzuimpercentage",
            min=decimal.Decimal(0), max=decimal.Decimal(100),
            required=False)
    form.widget(absentee_percentage="tno.euphorie.company.TextSpan1")
    accidents = schema.Int(
            title=u"Aantal ongevallen vorig jaar",
            required=False)
    form.widget(accidents="tno.euphorie.company.TextSpan1")
    incapacitated_workers = schema.Int(
            title=u"Aantal mensen in de WIA vorig jaar",
            required=False)
    form.widget(incapacitated_workers="tno.euphorie.company.TextSpan1")
    arbo_expert = schema.TextLine(
            title=u"Gegevens arbodienst/-deskundige",
            max_length=128,
            required=False)
    works_council_approval = schema.Date(
            title=u"Datum van akkoord OR/medewerkersvertegenwoordiging",
            min=datetime.date(2000,1,1),
            required=False)



class Company(GenericCompany):
    """Update the company details.

    This view is registered for :py:class:`PathGhost` instead of
    :py:obj:`euphorie.content.survey.ISurvey` since the
    :py:class:`SurveyPublishTraverser` generates a `PathGhost` object for
    the *inventory* component of the URL.
    """
    grok.context(PathGhost)
    grok.require("euphorie.client.ViewSurvey")
    grok.layer(ITnoReportPhaseSkinLayer)
    grok.template("report_company")
    grok.name("company")

    schema = DutchCompanySchema
    company = None
    errors = {}

    def _assertCompany(self):
        if self.company is not None:
            return
        session=SessionManager.session
        if session.dutch_company is None:
            session.dutch_company=DutchCompany(submit_date=datetime.date.today())
        directlyProvides(session.dutch_company, DutchCompanySchema)
        self.company=session.dutch_company





import logging

log = logging.getLogger(__name__)


def updateAbsenteePercentage(context):
    from z3c.saconfig import Session
    from euphorie.deployment.upgrade.utils import ColumnType
    from zope.sqlalchemy import datamanager

    session=Session()
    if ColumnType(session, "dutch_company", "absentee_percentage")=="numeric(5,2)":
        return

    log.info("Changing type for dutch_company.absentee_percentage to NUMERIC(5,2)")
    session.execute("ALTER TABLE dutch_company ALTER COLUMN absentee_percentage TYPE NUMERIC(5,2);")
    datamanager.mark_changed(session)




from sqlalchemy import sql
from five import grok
from euphorie.client import model
from euphorie.client.survey import PathGhost
from euphorie.client.survey import ActionPlan
from euphorie.client.survey import SurveyPublishTraverser
from tno.euphorie.interfaces import ITnoIdentificationPhaseSkinLayer
from tno.euphorie.interfaces import ITnoEvaluationPhaseSkinLayer
from tno.euphorie.interfaces import ITnoActionPlanPhaseSkinLayer
from tno.euphorie.interfaces import ITnoReportPhaseSkinLayer

class TnoSurveyPublishTraverser(SurveyPublishTraverser):
    phases = {
            'identification': ITnoIdentificationPhaseSkinLayer,
            'evaluation': ITnoEvaluationPhaseSkinLayer,
            'actionplan': ITnoActionPlanPhaseSkinLayer,
            'report': ITnoReportPhaseSkinLayer,
            }


grok.templatedir('templates')


class TnoActionPlanView(ActionPlan):
    grok.context(PathGhost)
    grok.require('euphorie.client.ViewSurvey')
    grok.layer(ITnoActionPlanPhaseSkinLayer)
    grok.template('actionplan')
    grok.name('index_html')

    question_filter = sql.or_(model.MODULE_WITH_RISK_FILTER,
                              model.RISK_PRESENT_FILTER)



from tno.euphorie.testing import TnoEuphorieTestCase


class TestingTests(TnoEuphorieTestCase):
    def testTnoEuphorieInstalled(self):
        quickinstaller=self.portal.portal_quickinstaller
        self.assertEqual(
                quickinstaller.isProductInstalled("tno.euphorie"),
                True)

    def testClientUserCataloged(self):
        self.assertNotEqual(
                self.portal.acl_users.getUserById("client"), None)





from sqlalchemy import sql
from five import grok
from euphorie.client import model
from euphorie.client.module import ActionPlanView
from tno.euphorie.interfaces import ITnoActionPlanPhaseSkinLayer

grok.templatedir('templates')


class TnoActionPlanView(ActionPlanView):
    grok.context(model.Module)
    grok.require('euphorie.client.ViewSurvey')
    grok.layer(ITnoActionPlanPhaseSkinLayer)
    grok.name('index_html')

    question_filter = sql.and_(
            model.RISK_OR_MODULE_WITH_DESCRIPTION_FILTER,
            sql.or_(model.MODULE_WITH_RISK_FILTER,
                    model.RISK_PRESENT_FILTER))



from tno.euphorie import testing

NORMAL_SURVEY =  \
        """<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
             <title>ICT</title>
             <survey>
              <title>Software development</title>
              <module optional="no">
                <title>Module one</title>
                <description>Hello</description>
                 <risk type="risk">
                   <title>New hires are not aware of design patterns.</title>
                   <description>&lt;p&gt;Every developer should know about them..&lt;/p&gt;</description>
                   <evaluation-method>direct</evaluation-method>
                   <image caption="Key image" content-type="image/gif">R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAEALAAAAAABAAEAAAIBTAA7</image>
                 </risk>
              </module>
            </survey>
          </sector>"""

TOP5_SURVEY =  \
        """<sector xmlns="http://xml.simplon.biz/euphorie/survey/1.0">
             <title>ICT</title>
             <survey>
              <title>Software development</title>
              <module optional="no">
                <title>Module one</title>
                <description>Hello</description>
                 <risk type="top5">
                   <title>New hires are not aware of design patterns.</title>
                   <description>&lt;p&gt;Every developer should know about them..&lt;/p&gt;</description>
                   <evaluation-method>direct</evaluation-method>
                   <image caption="Key image" content-type="image/gif">R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAEALAAAAAABAAEAAAIBTAA7</image>
                 </risk>
              </module>
            </survey>
          </sector>"""


class ActionPlanBrowserTests(testing.TnoEuphorieFunctionalTestCase):
    BASE_URL = "http://nohost/plone/client/nl?language=nl-NL"

    def createSurvey(self, survey):
        from euphorie.client.tests.utils import addSurvey
        self.loginAsPortalOwner()
        addSurvey(self.portal, survey)

    def startSurveySession(self):
        from Products.Five.testbrowser import Browser
        browser=Browser()
        browser.open(self.BASE_URL)
        # Register a new user
        testing.registerUserInClient(browser)
        # Create a new survey session
        browser.getControl(name="title:utf8:ustring").value="Test session"
        browser.getControl(name="next", index=1).click()
        # Start the survey
        browser.getForm().submit()
        browser.handleErrors=False
        return browser

    def test_skip_top5_risk_if_not_present(self):
        # This is a deviation from standard Euphorie which always asks for
        # policy and top5 risks.
        self.createSurvey(TOP5_SURVEY)
        browser = self.startSurveySession()
        # Jump to the risk and note that it is not present
        browser.getLink('Start risico inventarisatie').click()
        url = browser.url
        browser.getControl(name='next', index=1).click()
        browser.getControl(name='answer').value = ['yes']
        browser.getControl('next').click()
        # Now go to action plan view of the module make sure its next step is
        # the report landing page.
        url = url.replace('identification', 'actionplan')
        browser.open(url)
        self.assertEqual(
                browser.getLink('Volgende').url,
                'http://nohost/plone/client/nl/ict/software-development/report')




import decimal
from five import grok
from tno.euphorie.model import DutchCompany
from tno.euphorie.module import TnoActionPlanView as TnoModuleActionPlan
from tno.euphorie.risk import TnoActionPlanView as TnoRiskActionPlan
from tno.euphorie.company import DutchCompanySchema
from tno.euphorie.interfaces import ITnoClientAPISkinLayer
from tno.euphorie.report import TnoActionPlanReportDownload
from euphorie.client.api.module import ActionPlan as ModuleActionPlan
from euphorie.client.api.risk import ActionPlan as RiskActionPlan
from euphorie.client.api import JsonView
try:
    from euphorie.json import get_json_date
    from euphorie.json import get_json_int
    from euphorie.json import get_json_string
    from euphorie.json import get_json_token
    from euphorie.json import vocabulary_options
    from euphorie.json import vocabulary_token
except ImportError:
    from euphorie.client.api import get_json_date
    from euphorie.client.api import get_json_int
    from euphorie.client.api import get_json_string
    from euphorie.client.api import get_json_token
    from euphorie.client.api import vocabulary_options
    from euphorie.client.api import vocabulary_token
from euphorie.client.model import SurveySession


def apply_monkeys():
    RiskActionPlan.question_filter = TnoRiskActionPlan.question_filter
    ModuleActionPlan.question_filter = TnoModuleActionPlan.question_filter


def get_json_decimal(input, name, required=False, default=None):
    value = input.get(name)
    if value is None:
        if not required:
            return default
        raise KeyError('Required field %s is missing' % name)
    if not isinstance(value, (int, float)):
        raise ValueError('Field %s has wrong type' % name)
    return decimal.Decimal(str(value))  # Use str to prevent rounding issues


class Company(JsonView):
    grok.context(SurveySession)
    grok.layer(ITnoClientAPISkinLayer)
    grok.require('zope2.View')
    grok.name('company')

    def update(self):
        if self.context.dutch_company is None:
            self.context.dutch_company = DutchCompany()

    def do_GET(self):
        company = self.context.dutch_company
        return {'type': 'dutch-company',
                'title': company.title,
                'visit-address': {
                    'address': company.address_visit_address,
                    'postal': company.address_visit_postal,
                    'city': company.address_visit_city,
                },
                'postal-address': {
                    'address': company.address_postal_address,
                    'postal': company.address_postal_postal,
                    'city': company.address_postal_city,
                },
                'email': company.email,
                'phone': company.phone,
                'activity': company.activity,
                'department': company.department,
                'location': company.location,
                'employees': vocabulary_token(
                    DutchCompanySchema['employees'], company.employees)
                    if company.employees else None,
                'employees-options': vocabulary_options(
                    DutchCompanySchema['employees'], self.request),
                'absentee-percentage': float(company.absentee_percentage)
                    if company.absentee_percentage is not None else None,
                'accidents': company.accidents,
                'incapacitated-workers': company.incapacitated_workers,
                'submitter': {
                    'name': company.submitter_name,
                    'function': company.submitter_function,
                },
                'submitted': company.submit_date.isoformat()
                        if company.submit_date is not None else None,
                'arbo-expert': company.arbo_expert,
                'works-council-approval':
                    company.works_council_approval.isoformat()
                    if company.works_council_approval is not None else None,
                }

    def do_PUT(self):
        company = self.context.dutch_company
        try:
            company.title = get_json_string(self.input, 'title',
                    default=company.title, length=128)
            if 'visit-address' in self.input:
                input = self.input['visit-address']
                company.address_visit_address = get_json_string(
                        input, 'address',
                        default=company.address_visit_address)
                company.address_visit_postal = get_json_string(
                        input, 'postal',
                        default=company.address_visit_postal, length=16)
                company.address_visit_city = get_json_string(
                        input, 'city',
                        default=company.address_visit_city, length=64)
            if 'postal-address' in self.input:
                input = self.input['postal-address']
                company.address_postal_address = get_json_string(
                        input, 'address',
                        default=company.address_postal_address)
                company.address_postal_postal = get_json_string(
                        input, 'postal',
                        default=company.address_postal_postal, length=16)
                company.address_postal_city = get_json_string(
                        input, 'city',
                        default=company.address_postal_city, length=64)
            company.email = get_json_string(self.input, 'email',
                    default=company.email, length=128)
            company.phone = get_json_string(self.input, 'phone',
                    default=company.email, length=64)
            company.activity = get_json_string(self.input, 'activity',
                    default=company.activity, length=64)
            company.department = get_json_string(self.input, 'department',
                    default=company.department, length=64)
            company.location = get_json_string(self.input, 'location',
                    default=company.location, length=64)
            company.submit_date = get_json_date(self.input, 'submitted',
                    default=company.submit_date)
            company.employees = get_json_token(self.input, 'employees',
                    DutchCompanySchema['employees'], company.employees)
            company.absentee_percentage = get_json_decimal(self.input,
                    'absentee-percentage', default=company.absentee_percentage)
            company.accidents = get_json_int(self.input, 'accidents',
                    default=company.accidents)
            company.incapacitated_workers = get_json_int(self.input,
                    'incapacitated-workers',
                    default=company.incapacitated_workers)
            if 'submitter' in self.input:
                input = self.input['submitter']
                company.submitter_name = get_json_string(input,
                        'name', default=company.submitter_name, length=64)
                company.submitter_function = get_json_string(input,
                        'function', default=company.submitter_function,
                        length=64)
            company.arbo_expert = get_json_string(self.input, 'arbo-expert',
                    default=company.arbo_expert, length=128)
            company.works_council_approval = get_json_date(self.input,
                    'works-council-approval',
                    default=company.works_council_approval)
        except (KeyError, ValueError) as e:
            return {'type': 'error',
                    'message': str(e)}
        return self.do_GET()


class ActionPlanReport(grok.View):
    grok.context(SurveySession)
    grok.require('zope2.View')
    grok.layer(ITnoClientAPISkinLayer)
    grok.name('report-actionplan')

    def update(self):
        if self.context.dutch_company is None:
            self.context.dutch_company = DutchCompany()

    def render(self):
        view = TnoActionPlanReportDownload(self.request.survey, self.request)
        view.session = self.context
        return view.render()


_applied_monkeys = False
if not _applied_monkeys:
    apply_monkeys()
    _applied_monkeys = True



from sqlalchemy import sql
from five import grok
from euphorie.client import model
from euphorie.client.risk import ActionPlanView
from tno.euphorie.interfaces import ITnoActionPlanPhaseSkinLayer

grok.templatedir('templates')


class TnoActionPlanView(ActionPlanView):
    grok.context(model.Risk)
    grok.layer(ITnoActionPlanPhaseSkinLayer)
    grok.name('index_html')
    grok.template('risk_actionplan')
    grok.require('euphorie.client.ViewSurvey')

    question_filter = sql.or_(model.MODULE_WITH_RISK_FILTER,
                              model.RISK_PRESENT_FILTER)



from tno.euphorie import testing

class CompanyBrowserTests(testing.TnoEuphorieFunctionalTestCase):
    BASE_URL = "http://nohost/plone/client/nl?language=nl-NL"

    def createSurvey(self):
        from euphorie.content.tests.utils import BASIC_SURVEY
        from euphorie.client.tests.utils import addSurvey
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)

    def startSurveySession(self):
        from Products.Five.testbrowser import Browser
        browser=Browser()
        browser.open(self.BASE_URL)
        # Register a new user
        testing.registerUserInClient(browser)
        # Create a new survey session
        browser.getControl(name="title:utf8:ustring").value="Test session"
        browser.getControl(name="next", index=1).click()
        # Start the survey
        browser.getForm().submit()
        browser.handleErrors=False
        return browser

    def testDutchCompanyFormUsed(self):
        self.createSurvey()
        browser=self.startSurveySession()
        # Jump to the report phase
        browser.getLink("Rapport").click()
        browser.getControl(name="next").click()
        # We should now be at the company form
        self.assertEqual(browser.url,
                "http://nohost/plone/client/nl/ict/software-development/report/company")
        self.assertTrue("Bezoekadres bedrijf" in browser.contents)

    def testDutchCompanyReportViewUsed(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/view")
        self.assertTrue("Bezoekadres bedrijf" in browser.contents)

    def testDutchCompanyReportDownloadUsed(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/download")
        self.assertTrue("Bezoekadres bedrijf" in browser.contents)

    def testDecimalAbsenteePercentage_DutchNotation(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="50,1"
        browser.getControl(name="form.buttons.next").click()
        self.assertEqual(browser.url,
                "http://nohost/plone/client/nl/ict/software-development/report/view")

    def testDecimalAbsenteePercentage_EnglishNotation(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="40.1"
        browser.getControl(name="form.buttons.next").click()
        self.assertTrue("Vul een getal (maximaal 100) in." in browser.contents)

    def testInvalidAbsenteePercentageGetsErrorMessage(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="4.0.1"
        browser.getControl(name="form.buttons.next").click()
        self.assertTrue("Vul een getal (maximaal 100) in." in browser.contents)

    def testDecimalAbsenteePercentageNotRoundedInReport(self):
        # Test for http://code.simplon.biz/tracker/tno-euphorie/ticket/162
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="50,1"
        browser.getControl(name="form.buttons.next").click()
        self.assertTrue("50,1" in browser.contents)

    def testPartialYear(self):
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.submit_date-day").value="10"
        browser.getControl(name="form.widgets.submit_date-month").value=["9"]
        browser.getControl(name="form.widgets.submit_date-year").value="8"
        browser.getControl(name="form.buttons.next").click()
        self.assertTrue("Geef een datum na 1 januari 2000 op." in browser.contents)
        browser.getControl(name="form.widgets.submit_date-year").value="2008"
        browser.getControl(name="form.buttons.next").click()
        self.assertEqual(browser.url,
                "http://nohost/plone/client/nl/ict/software-development/report/view")
        self.assertTrue("10 september 2008" in browser.contents)

    def testEmployeeSaved(self):
        # Test for http://code.simplon.biz/tracker/tno-euphorie/ticket/151
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.employees").value=["over25"]
        browser.getControl(name="form.buttons.next").click()
        self.assertEqual(browser.url,
                "http://nohost/plone/client/nl/ict/software-development/report/view")
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        self.assertEqual(
            browser.getControl(name="form.widgets.employees").value,
            ["over25"])

    def testWorksCouncilApprovalNotSetAfterOtherError(self):
        # Test for http://code.simplon.biz/tracker/tno-euphorie/ticket/163
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="ABC"
        browser.getControl(name="form.buttons.next").click()
        self.assertEqual(browser.getControl(name="works_council").value, [])

    def testAbsenteePercentageNotLost(self):
        # Test for http://code.simplon.biz/tracker/tno-euphorie/ticket/167
        self.createSurvey()
        browser=self.startSurveySession()
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        browser.getControl(name="form.widgets.absentee_percentage").value="50"
        browser.getControl(name="form.buttons.next").click()
        self.assertEqual(browser.url,
                "http://nohost/plone/client/nl/ict/software-development/report/view")
        browser.open("http://nohost/plone/client/nl/ict/software-development/report/company")
        value = browser.getControl(
                name="form.widgets.absentee_percentage").value
        self.assertTrue(value.startswith('50'))



import unittest
from tno.euphorie.testing import TnoEuphorieFunctionalTestCase


class get_json_decimal_tests(unittest.TestCase):
    def get_json_decimal(self, *a, **kw):
        from ..api import get_json_decimal
        return get_json_decimal(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(KeyError,
                self.get_json_decimal, {}, 'field', required=True)

    def test_missing_optional_field(self):
        self.assertEqual(
                self.get_json_decimal({}, 'field', required=False,
                    default='default'),
                'default')

    def test_bad_type(self):
        self.assertRaises(ValueError,
                self.get_json_decimal, {'field': 'dummy'}, 'field')

    def test_proper_value(self):
        import decimal
        self.assertEqual(
                self.get_json_decimal({'field': 15}, 'field'),
                decimal.Decimal('15'))

    def test_rounding(self):
        import decimal
        self.assertEqual(
                self.get_json_decimal({'field': 25.6}, 'field'),
                decimal.Decimal('25.6'))


class ViewBrowserTests(TnoEuphorieFunctionalTestCase):
    def test_company_no_date_present(self):
        import datetime
        import json
        from z3c.saconfig import Session
        from euphorie.client.model import SurveySession
        from euphorie.content.tests.utils import BASIC_SURVEY
        from euphorie.client.tests.utils import addAccount
        from euphorie.client.tests.utils import addSurvey
        from euphorie.client.api.authentication import generate_token
        from Products.Five.testbrowser import Browser
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        account = addAccount(password='secret')
        survey_session = SurveySession(
                title=u'Dummy session',
                created=datetime.datetime(2012, 4, 22, 23, 5, 12),
                modified=datetime.datetime(2012, 4, 23, 11, 50, 30),
                zodb_path='nl/ict/software-development',
                account=account)
        Session.add(survey_session)
        browser = Browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open(
                'http://nohost/plone/client/api/users/1/sessions/1/company')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(response['type'], 'dutch-company')


class CompanyTests(TnoEuphorieFunctionalTestCase):
    def Company(self, *a, **kw):
        from tno.euphorie.api import Company
        return Company(*a, **kw)

    def create_context(self):
        import datetime
        from euphorie.client.model import SurveySession
        from tno.euphorie.model import DutchCompany
        company = DutchCompany(title=u'Acme B.V.',
                employees='40h',
                email='john@example.com',
                submit_date=datetime.date(2012, 6, 6))
        return SurveySession(dutch_company=company)

    def test_do_GET_result(self):
        import json
        from zope.publisher.browser import TestRequest
        context = self.create_context()
        view = self.Company(context, TestRequest())
        response = view.do_GET()
        self.assertTrue(isinstance(response, dict))
        self.assertEqual(
                set(response),
                set(['type', 'title', 'visit-address', 'postal-address',
                     'email', 'phone', 'activity', 'submitter', 'department',
                     'location', 'submitted', 'employees', 'employees-options',
                     'absentee-percentage', 'accidents',
                     'incapacitated-workers', 'arbo-expert',
                     'works-council-approval']))
        json.dumps(response)  # Check it is serializable
        self.assertEqual(response['type'], 'dutch-company')
        self.assertEqual(response['title'], u'Acme B.V.')
        self.assertEqual(response['absentee-percentage'], None)
        self.assertEqual(response['employees'], '40h')
        self.assertEqual(response['submitted'], '2012-06-06')

    def test_do_GET_with_absentee_percentage(self):
        import decimal
        import json
        from zope.publisher.browser import TestRequest
        context = self.create_context()
        context.dutch_company.absentee_percentage = decimal.Decimal('12.34')
        view = self.Company(context, TestRequest())
        response = view.do_GET()
        json.dumps(response)  # Check it is serializable
        self.assertEqual(response['absentee-percentage'], 12.34)

    def test_do_PUT(self):
        import datetime
        import decimal
        from zope.publisher.browser import TestRequest
        context = self.create_context()
        view = self.Company(context, TestRequest())
        view.input = {'visit-address': {
                          'address': u'Dorpsstraat 15',
                          'city': u'Ons Dorp'},
                      'department': u'Incasso',
                      'accidents': 15,
                      'absentee-percentage': 25.6,
                      'employees': 'over25',
                      'works-council-approval': '2012-06-04',
                     }
        response = view.do_PUT()
        self.assertEqual(response['type'], 'dutch-company')
        self.assertEqual(
                context.dutch_company.address_visit_address,
                u'Dorpsstraat 15')
        self.assertEqual(
                context.dutch_company.address_visit_city,
                u'Ons Dorp')
        self.assertEqual(context.dutch_company.department, u'Incasso')
        self.assertEqual(context.dutch_company.accidents, 15)
        self.assertEqual(
                context.dutch_company.absentee_percentage,
                decimal.Decimal('25.6'))
        self.assertEqual(context.dutch_company.employees, 'over25')
        self.assertEqual(
                context.dutch_company.works_council_approval,
                datetime.date(2012, 6, 4))


class ActionPlanReportTests(TnoEuphorieFunctionalTestCase):
    def test_browser_get(self):
        import datetime
        from z3c.saconfig import Session
        from euphorie.client.model import SurveySession
        from euphorie.content.tests.utils import BASIC_SURVEY
        from euphorie.client.tests.utils import addAccount
        from euphorie.client.tests.utils import addSurvey
        from euphorie.client.api.authentication import generate_token
        from tno.euphorie.model import DutchCompany
        from Products.Five.testbrowser import Browser
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        account = addAccount(password='secret')
        survey_session = SurveySession(
                title=u'Dummy session',
                created=datetime.datetime(2012, 4, 22, 23, 5, 12),
                modified=datetime.datetime(2012, 4, 23, 11, 50, 30),
                zodb_path='nl/ict/software-development',
                account=account)
        survey_session.dutch_company = DutchCompany(
                title=u'Acme B.V.',
                employees='40h',
                email='john@example.com',
                submit_date=datetime.date(2012, 6, 6))
        Session.add(survey_session)
        browser = Browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.handleErrors = False
        browser.open(
                'http://nohost/plone/client/api/users/1/sessions/1/'
                'report-actionplan')
        self.assertEqual(browser.headers['Content-Type'], 'application/rtf')
        self.assertTrue('Bedrijfsnaam' in browser.contents)



from sqlalchemy import orm
from sqlalchemy.sql import functions
from sqlalchemy import schema
from sqlalchemy import types
from euphorie.client.model import BaseObject
from euphorie.client.enum import Enum


class DutchCompany(BaseObject):
    """Information about a Dutch company."""
    __tablename__ = "dutch_company"

    id = schema.Column(types.Integer(), primary_key=True, autoincrement=True)
    session_id = schema.Column(types.Integer(),
        schema.ForeignKey("session.id", onupdate="CASCADE", ondelete="CASCADE"),
        nullable=False, index=True)
    session = orm.relation("SurveySession",
            cascade="all,delete-orphan", single_parent=True,
            backref=orm.backref("dutch_company", uselist=False, cascade="all"))

    title = schema.Column(types.Unicode(128))
    address_visit_address = schema.Column(types.UnicodeText())
    address_visit_postal = schema.Column(types.Unicode(16))
    address_visit_city = schema.Column(types.Unicode(64))
    address_postal_address = schema.Column(types.UnicodeText())
    address_postal_postal = schema.Column(types.Unicode(16))
    address_postal_city = schema.Column(types.Unicode(64))
    email = schema.Column(types.String(128))
    phone = schema.Column(types.String(32))
    activity = schema.Column(types.Unicode(64))
    submitter_name = schema.Column(types.Unicode(64))
    submitter_function = schema.Column(types.Unicode(64))
    department = schema.Column(types.Unicode(64))
    location = schema.Column(types.Unicode(64))
    submit_date = schema.Column(types.Date(), default=functions.now())
    employees = schema.Column(Enum([None, "40h", "max25", "over25"]))
    absentee_percentage = schema.Column(types.Numeric(precision=5, scale=2))
    accidents = schema.Column(types.Integer())
    incapacitated_workers = schema.Column(types.Integer())
    arbo_expert = schema.Column(types.Unicode(128))
    works_council_approval = schema.Column(types.Date())


_instrumented = False
if not _instrumented:
    from sqlalchemy.ext import declarative
    from euphorie.client import model
    declarative.instrument_declarative(DutchCompany, model.metadata._decl_registry, model.metadata)
    _instrumented = True



from tno.euphorie.testing import TnoEuphorieFunctionalTestCase


class ClientPublishTraverserTests(TnoEuphorieFunctionalTestCase):
    def ClientPublishTraverser(self, *a, **kw):
        from tno.euphorie.client import ClientPublishTraverser
        return ClientPublishTraverser(*a, **kw)

    def test_api_access(self):
        from Products.Five.testbrowser import Browser
        browser = Browser()
        browser.open('http://nohost/plone/client/api')
        self.assertEqual(
                browser.headers['Content-Type'],
                'application/json')



import datetime
import transaction
from Acquisition import aq_base
from Acquisition import aq_inner
from plone.directives import form
from five import grok
import lxml.etree
import lxml.objectify
from z3c.saconfig import Session
from zope.interface import Invalid
from z3c.form import button
from z3c.form.interfaces import WidgetActionExecutionError
from plone.namedfile.field import NamedFile
from euphorie.client.country import IClientCountry
from euphorie.client.sector import IClientSector
from euphorie.client.profile import BuildSurveyTree
from tno.euphorie.interfaces import ITnoClientSkinLayer
from euphorie.client import model
from euphorie.client.navigation import FindFirstQuestion
from euphorie.client.navigation import QuestionURL
from euphorie.content.interfaces import IQuestionContainer
from euphorie.content.survey import ISurvey
from euphorie.content.profilequestion import IProfileQuestion
from euphorie.content.upload import el_unicode
from euphorie.content.upload import attr_unicode
from euphorie.client.session import SessionManager
from tno.euphorie.model import DutchCompany
from tno.euphorie import MessageFactory as _

grok.templatedir("templates")


def parse_date(value, default=None):
    try:
        return datetime.datetime.strptime(value, "%d/%m/%Y").date()
    except ValueError:
        return default


def attr_date(node, tag, default=None):
    value = unicode(node.attrib.get(tag, u"")).strip()
    if not value:
        return default
    return parse_date(value, default)


def attr_int(node, tag, default=None):
    value = node.attrib.get(tag, None)
    if value is None:
        return default
    try:
        return int(value)
    except ValueError:
        return default


class UploadSchema(form.Schema):
    file = NamedFile(
            title=_("label_session_file", default=u"RI&E bestand"),
            required=True)


class Upload(form.SchemaForm):
    grok.context(IClientCountry)
    grok.require("euphorie.client.ViewSurvey")
    grok.layer(ITnoClientSkinLayer)
    grok.name("rie-session")
    grok.template("upload")
    form.wrap(False)

    ignoreContext = True
    schema = UploadSchema

    def updateCompany(self, input, session):
        mapping = {"afdeling": ("department", attr_unicode),
                   "bezoekadres": ("address_visit_address", attr_unicode),
                   "bezoekpostcode": ("address_visit_postal", attr_unicode),
                   "bezoekwoonplaats": ("address_visit_city", attr_unicode),
                   "postadres": ("address_postal_address", attr_unicode),
                   "postcode": ("address_postal_postal", attr_unicode),
                   "woonplaats": ("address_postal_city", attr_unicode),
                   "emailadres": ("email", attr_unicode),
                   "telefoonnummer": ("phone", attr_unicode),
                   "auteur": ("submitter_name", attr_unicode),
                   "datum": ("submit_date", attr_date),
                   "bedrijfsactiviteit": ("activity", attr_unicode),
                   "invullerfunctie": ("submitter_function", attr_unicode),
                   "lokatie": ("location", attr_unicode),
                   "verzuimpercentage": ("absentee_percentage", attr_int),
                   "ongevallen": ("accidents", attr_int),
                   "WAO": ("incapacitated_workers", attr_int),
                   }
        data = input.gegevens
        company = session.dutch_company = DutchCompany()

        for (old, new) in mapping.items():
            setattr(company, new[0], new[1](data, old))

        employees = attr_int(data, "aantalindienst")
        if employees is not None:
            if employees <= 1:
                company.employees = "40h"
            elif 1 < employees <= 25:
                company.employees = "max25"
            elif 25 < employees:
                company.employees = "over25"

        if attr_int(data, "orakkoord") == 1:
            company.works_council_approved = attr_date(data, "orakkoorddatum")

    def buildProfile(self, input, survey, session):
        idmap = {}
        for profilequestion in survey.values():
            if not IProfileQuestion.providedBy(profilequestion):
                continue
            idmap[profilequestion.external_id] = profilequestion
        if not idmap:
            return ({}, {})

        profile = {}
        keuzemap = {}  # Map `keuze' to profile index
        for facet in input.profiel.facet:
            question = idmap.get(facet.attrib["vraag-id"])
            if question is None:
                continue
            if question.type == "optional":
                profile[question.id] = True
                keuzemap[facet.keuze.attrib["antwoord"]] = 0
            elif question.type == "repeat":
                profile[question.id] = []
                for (i, keuze) in enumerate(facet.keuze):
                    antwoord = attr_unicode(keuze, "antwoord")
                    profile[question.id].append(antwoord)
                    keuzemap[keuze.attrib["antwoord"]] = i

        return (profile, keuzemap)

    def buildExternalIdMap(self, root, zodb_path=[], mapping=None):
        if mapping is None:
            mapping = {}
        for child in root.values():
            external_id = getattr(aq_base(child), "external_id", None)
            newpath = zodb_path + [child.id]
            if external_id is not None:
                mapping[external_id] = "/".join(newpath)
            if IQuestionContainer.providedBy(child):
                self.buildExternalIdMap(child, newpath, mapping)
        return mapping

    def updateAnswers(self, input, keuzemap, survey, session):
        idmap = self.buildExternalIdMap(survey)
        query = Session.query(model.Risk)\
                .filter(model.Risk.session_id == session.id)
        identification_map = {"1": "yes",
                              "2": "no",
                              "3": "n/a"}
        priority_map = {"laag": "low",
                        "midden": "medium",
                        "hoog": "high"}

        for antwoord in input.antwoorden.antwoord:
            risk_id = idmap.get(antwoord.attrib["risk-id"])
            if risk_id is None:
                continue

            risk = query\
                .filter(model.Risk.zodb_path == risk_id)\
                .filter(model.Risk.profile_index ==
                        keuzemap.get(antwoord.attrib["keuze"], 0))\
                .first()
            if risk is None:
                continue

            risk.identification = identification_map.get(antwoord.attrib["inventariseren"])
            if antwoord.attrib["inventariseren"] == "-1":
                risk.postponed = True
            elif risk.identification is not None:
                risk.postponed = False
            risk.probability = attr_int(antwoord, "evalueren1")
            risk.frequency = attr_int(antwoord, "evalueren2")
            risk.effect = attr_int(antwoord, "evalueren3")
            risk.priority = priority_map.get(antwoord.attrib["prioriteit"])
            if hasattr(antwoord, "opmerking"):
                risk.comment = el_unicode(antwoord, "opmerking")
            for pva in antwoord.iterchildren("pva"):
                plan = model.ActionPlan(risk=risk)
                plan.action_plan = el_unicode(pva, "maatregel")
                plan.prevention_plan = el_unicode(pva, "preventietaken")
                plan.requirements = el_unicode(pva, "preventiekennis")
                plan.responsible = el_unicode(pva, "uitvoerder")
                try:
                    plan.budget = int(pva.budget.text)
                except (TypeError, ValueError):
                    pass
                timeline = pva.planning.text.split()
                plan.planning_start = parse_date(timeline[0])
                plan.planning_end = parse_date(timeline[2])
                Session.add(plan)

        session.touch()

    def parseInput(self, input):
        """Try to parse a RI&E session file. The parsed data is
        returned as a lxml.objectified element. If the file could
        not be parsed or has the wrong format None is returned instead.
        """
        try:
            input = lxml.objectify.fromstring(input)
        except lxml.etree.XMLSyntaxError:
            return None

        if input.tag != "rieprogress":
            return None

        rie_path = input.attrib["rie_path"]
        if not rie_path.startswith("/rie/data/"):
            return None

        return input

    def findSurvey(self, input):
        """Find the survey to match the (already parsed) input data."""
        rie = input.attrib["rie_path"].split("/")[3]
        matches = []
        for sector in aq_inner(self.context).values():
            if not IClientSector.providedBy(sector):
                continue
            for survey in sector.values():
                if ISurvey.providedBy(survey) and \
                        survey.id != 'preview' and \
                        getattr(aq_base(survey), "external_id", None) == rie:
                    matches.append(survey)
        if not matches:
            return None
        # Pick the oldest published survey on the assumption this is not a
        # modified copy.
        matches.sort(key=lambda s: s.published[2], reverse=True)
        return matches[0]

    @button.buttonAndHandler(_("button_upload", default=u"Upload"), name="upload")
    def handleUpload(self, action):
        (data, errors) = self.extractData()
        if errors:
            return

        input = self.parseInput(data["file"].data)
        if input is None:
            raise WidgetActionExecutionError("file",
                    Invalid(_("error_invalid_session_file",
                        default=u"Geen valide RI&E bestand.")))

        survey = self.findSurvey(input)
        if survey is None:
            raise WidgetActionExecutionError("file",
                    Invalid(_("error_unknown_survey",
                        default=u"De gebruikte vragenlijst bestaat niet op deze site.")))

        session = SessionManager.start(
                attr_unicode(input, "rienaam", u"RI&E import"), survey)
        self.updateCompany(input, session)
        (profile, keuzemap) = self.buildProfile(input, survey, session)
        BuildSurveyTree(survey, profile, session)
        self.updateAnswers(input, keuzemap, survey, session)

        question = FindFirstQuestion(dbsession=session)
        if question is None:
            transaction.get().doom()
            raise WidgetActionExecutionError("file",
                    Invalid(u"Deze RI&E is helaas teveel veranderd om te kunnen gebruiken."))
        self.request.response.redirect(
                QuestionURL(survey, question, phase="identification"))



from setuptools import setup, find_packages
import os

version = '4.3'

setup(name="tno.euphorie",
      version=version,
      description="TNO specific extensions for Euphorie",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "changes.rst")).read(),
      classifiers=[
          "Framework :: Plone",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Natural Language :: Dutch",
          "Operating System :: OS Independent",
          "Programming Language :: Python :: 2 :: Only",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Topic :: Internet :: WWW/HTTP",
        ],
      keywords="Euphorie OIRA RIE",
      author="Wichert Akkerman",
      author_email="wichert@simplon.biz",
      url="http://readthedocs.org/docs/tnoeuphorie/en/latest/",
      license="GPL",
      packages=find_packages("src"),
      namespace_packages=["tno"],
      include_package_data=True,
      package_dir={"": "src"},
      zip_safe=False,
      install_requires=[
          "Euphorie >=6.0",
          "plone.browserlayer",
          "five.grok",
          "plone.directives.form",
          "plone.namedfile",
          "setuptools",
          "z3c.form",
          "zope.i18nmessageid",
          "z3c.appconfig",
      ],
      tests_require=[
          "Euphorie [tests]",
      ],
      extras_require={
          "tests": ["Euphorie [tests]"],
      },
      )



from rtfng.document.paragraph import Paragraph
from rtfng.document.paragraph import Cell
from rtfng.document.paragraph import Table
from rtfng.PropertySets import TabPropertySet
from sqlalchemy import sql
from z3c.saconfig import Session
from five import grok
from zope.i18n import translate
from plonetheme.nuplone.utils import formatDate
from euphorie.client import model
from euphorie.client.report import ActionPlanReportView
from euphorie.client.report import ActionPlanReportDownload
from euphorie.client.report import createSection
from euphorie.client import MessageFactory as eu_
from tno.euphorie.interfaces import ITnoReportPhaseSkinLayer
from tno.euphorie.model import DutchCompany
from tno.euphorie.company import DutchCompanySchema

grok.templatedir("templates")

def formatAddress(address, postal, city):
    output=[]
    if address:
        output.append(address)
        if postal or city:
            output.append(u"\n")
    bits=filter(None, [postal, city])
    if bits:
        output.append(u" ".join(bits))
    return u"".join(output) if output else None



class TnoActionPlanReportView(ActionPlanReportView):
    grok.layer(ITnoReportPhaseSkinLayer)
    grok.name("view")
    grok.template("report_actionplan")

    def getNodes(self):
        query = Session.query(model.SurveyTreeItem)\
                .filter(model.SurveyTreeItem.session == self.session)\
                .filter(sql.not_(model.SKIPPED_PARENTS))\
                .filter(sql.or_(model.MODULE_WITH_RISK_FILTER,
                                model.RISK_PRESENT_FILTER))\
                .order_by(model.SurveyTreeItem.path)
        return  query.all()

    def update(self):
        super(TnoActionPlanReportView, self).update()
        if self.session.dutch_company is None:
            self.session.dutch_company=DutchCompany()



class TnoActionPlanReportDownload(ActionPlanReportDownload):
    grok.layer(ITnoReportPhaseSkinLayer)
    grok.name("download")

    def getNodes(self):
        query = Session.query(model.SurveyTreeItem)\
                .filter(model.SurveyTreeItem.session == self.session)\
                .filter(sql.not_(model.SKIPPED_PARENTS))\
                .filter(sql.or_(model.MODULE_WITH_RISK_OR_TOP5_FILTER,
                                model.RISK_PRESENT_OR_TOP5_FILTER))\
                .order_by(model.SurveyTreeItem.path)
        return  query.all()

    def update(self):
        super(TnoActionPlanReportDownload, self).update()
        if self.session.dutch_company is None:
            self.session.dutch_company=DutchCompany()

    def addCompanyInformation(self, document):
        request=self.request
        company=self.session.dutch_company
        t=lambda txt: translate(txt, context=request)
        section = createSection(document, self.context, self.session,
                self.request)
        normal_style=document.StyleSheet.ParagraphStyles.Normal
        missing=t(eu_("missing_data", default=u"Not provided"))

        section.append(Paragraph(
            document.StyleSheet.ParagraphStyles.Heading1,
            t(eu_("plan_report_company_header", default=u"Company details"))))

        table=Table(TabPropertySet.DEFAULT_WIDTH*3, TabPropertySet.DEFAULT_WIDTH*8)

        field=DutchCompanySchema["title"]
        table.append(
                Cell(Paragraph(normal_style, str(field.title))),
                Cell(Paragraph(normal_style, company.title if company.title else missing)))

        address=formatAddress(company.address_visit_address,
                company.address_visit_postal, company.address_visit_city)
        table.append(
                Cell(Paragraph(normal_style, "Bezoekadres bedrijf")),
                Cell(Paragraph(normal_style, address if address else missing)))

        address=formatAddress(company.address_postal_address,
                company.address_postal_postal, company.address_postal_city)
        table.append(
                Cell(Paragraph(normal_style, "Postadres bedrijf")),
                Cell(Paragraph(normal_style, address if address else missing)))

        for key in ["email", "phone", "activity", "submitter_name",
                      "submitter_function", "department", "location"]:
            field=DutchCompanySchema[key]
            value=getattr(company, key, None)
            table.append(
                    Cell(Paragraph(normal_style, field.title)),
                    Cell(Paragraph(normal_style, value if value else missing))),

        formatDecimal=request.locale.numbers.getFormatter("decimal").format
        field=DutchCompanySchema["absentee_percentage"]
        table.append(
                Cell(Paragraph(normal_style, field.title)),
                Cell(Paragraph(normal_style, u"%s %%" % formatDecimal(company.absentee_percentage) if company.absentee_percentage else missing)))

        for key in [ "accidents", "incapacitated_workers"]:
            field=DutchCompanySchema[key]
            value=getattr(company, key, None)
            table.append(
                    Cell(Paragraph(normal_style, field.title)),
                    Cell(Paragraph(normal_style, "%d" % value if value is not None else missing)))

        field=DutchCompanySchema["submit_date"]
        table.append(
                Cell(Paragraph(normal_style, field.title)),
                Cell(Paragraph(normal_style, formatDate(request, company.submit_date) if company.submit_date else missing)))

        field=DutchCompanySchema["employees"]
        table.append(
                Cell(Paragraph(normal_style, field.title)),
                Cell(Paragraph(normal_style, field.vocabulary.getTerm(company.employees).title if company.employees else missing)))

        field=DutchCompanySchema["arbo_expert"]
        table.append(
                Cell(Paragraph(normal_style, str(field.title))),
                Cell(Paragraph(normal_style, company.arbo_expert if company.arbo_expert else missing)))

        section.append(table)



from zope.interface import Interface
from euphorie.client.interfaces import IClientSkinLayer
from euphorie.client.interfaces import IIdentificationPhaseSkinLayer
from euphorie.client.interfaces import IEvaluationPhaseSkinLayer
from euphorie.client.interfaces import IActionPlanPhaseSkinLayer
from euphorie.client.interfaces import IReportPhaseSkinLayer
from euphorie.client.api.interfaces import IClientAPISkinLayer


class IProductLayer(Interface):
    """Marker interface for requests indicating the tno.euphorie
    package has been installed.
    """


class ITnoClientSkinLayer(IClientSkinLayer):
    """Marker interface for the TNO client skin."""


class ITnoIdentificationPhaseSkinLayer(IIdentificationPhaseSkinLayer):
    """Marker interface for the identification phase in a tno.euphorie site."""


class ITnoEvaluationPhaseSkinLayer(IEvaluationPhaseSkinLayer):
    """Marker interface for the evaluation phase in a tno.euphorie site."""


class ITnoActionPlanPhaseSkinLayer(IActionPlanPhaseSkinLayer):
    """Marker interface for the action plan phase in a tno.euphorie site."""


class ITnoReportPhaseSkinLayer(IReportPhaseSkinLayer):
    """Marker interface for the report phase in a tno.euphorie site."""


class ITnoClientAPISkinLayer(IClientAPISkinLayer):
    """Marker itnerface for the tno.euphorie extensions to the client API."""



import unittest
from lxml import objectify
from tno.euphorie import testing


class parse_dateTests(unittest.TestCase):
    def parse_date(self, value, default=None):
        from tno.euphorie.session import parse_date
        return parse_date(value, default)

    def testMissingValue(self):
        self.assertEqual(self.parse_date(""), None)
        marker = []
        self.assertTrue(self.parse_date("", marker) is marker)

    def testInvalidValue(self):
        self.assertEqual(self.parse_date("invalid"), None)

    def testDutchDateOrder(self):
        import datetime
        self.assertEqual(self.parse_date("06/08/2010"),
                datetime.date(2010, 8, 6))


class attr_dateTests(unittest.TestCase):
    def attr_date(self, node, tag, default=None):
        from tno.euphorie.session import attr_date
        return attr_date(node, tag, default)

    def testMissingValue(self):
        from lxml import etree
        node = etree.Element("node")
        self.assertEqual(self.attr_date(node, "date"), None)
        marker = []
        self.assertTrue(self.attr_date(node, "date", marker) is marker)

    def testEmptyValue(self):
        from lxml import etree
        node = etree.Element("node", date=u"")
        self.assertEqual(self.attr_date(node, "date"), None)


class UploadTests(testing.TnoEuphorieTestCase):
    BASE_SNIPPET = """
            <rieprogress>
              <gegevens
                bezoekwoonplaats=""
                datum=""
                afdeling=""
                postcode=""
                organisatie=""
                deeltijdindienst=""
                arbodienstcontactpersoon=""
                bezoekadres=""
                emailadres=""
                bedrijfsactiviteit=""
                faxnummer=""
                invullerfunctie=""
                orakkoord=""
                telefoonnummer=""
                bezoekpostcode=""
                verzuimpercentage=""
                WAO=""
                arbodienstpostplaats=""
                arbodienstpostadres=""
                ongevallen=""
                auteur=""
                woonplaats=""
                arbodienstpostcode=""
                postadres=""
                arbodienstemail=""
                arbodienstnaam=""
                orakkoorddatum=""
                lokatie=""
                aantalindienst=""/>
            </rieprogress>"""

    def testUpdateCompany_Empty(self):
        from tno.euphorie.session import Upload
        from euphorie.client import model
        from tno.euphorie.model import DutchCompany
        session = model.SurveySession()
        input = objectify.fromstring(self.BASE_SNIPPET)
        view = Upload(None, None)
        view.updateCompany(input, session)
        self.assertTrue(isinstance(session.dutch_company, DutchCompany))

    def testUpdateCompany_AantalInDienst(self):
        from tno.euphorie.session import Upload
        from euphorie.client import model
        session = model.SurveySession()
        input = objectify.fromstring(self.BASE_SNIPPET)
        view = Upload(None, None)
        input.gegevens.attrib["aantalindienst"] = "1"
        view.updateCompany(input, session)
        self.assertEqual(session.dutch_company.employees, "40h")
        input.gegevens.attrib["aantalindienst"] = "25"
        view.updateCompany(input, session)
        self.assertEqual(session.dutch_company.employees, "max25")
        input.gegevens.attrib["aantalindienst"] = "999"
        view.updateCompany(input, session)
        self.assertEqual(session.dutch_company.employees, "over25")


class SessionBrowserTests(testing.TnoEuphorieFunctionalTestCase):
    def createSurvey(self):
        from euphorie.content.tests.utils import BASIC_SURVEY
        from euphorie.client.tests.utils import addSurvey
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)

    def createClientBrowser(self):
        from Products.Five.testbrowser import Browser
        from euphorie.client.tests.utils import registerUserInClient
        browser = Browser()
        browser.open(self.portal.client.nl["ict"]["software-development"].absolute_url())
        registerUserInClient(browser)
        return browser

    def testUploadFormIsMentionedOnSessionScreen(self):
        self.createSurvey()
        browser = self.createClientBrowser()
        browser.open(self.portal.client.nl.absolute_url())
        self.assertTrue(
                "Als u bestanden van de oude RI&amp;E omgeving heeft"
                in browser.contents)
        browser.getLink("dit formulier").click()
        self.assertEqual(
                browser.url,
                "http://nohost/plone/client/nl/rie-session")



import unittest

class FormatAddressTests(unittest.TestCase):
    def formatAddress(self, *a, **kw):
        from tno.euphorie.report import formatAddress
        return formatAddress(*a, **kw)

    def testEmptyAddress(self):
        self.assertEqual(self.formatAddress(None, None, None), None)

    def testAddressOnly(self):
        self.assertEqual(self.formatAddress(u"Street", None, None),
                u"Street")

    def testPostalOnly(self):
        self.assertEqual(self.formatAddress(None, u"Postal", None),
                u"Postal")

    def testPostalAndCity(self):
        self.assertEqual(self.formatAddress(None, u"Postal", u"City"),
                u"Postal City")

    def testFull(self):
        self.assertEqual(self.formatAddress(u"Street", u"Postal", u"City"),
                u"Street\nPostal City")
